"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var fs = require("fs");
var Datauri = require('datauri');
var func = require('rework-plugin-function');
var NAME = 'rework-base64';
/**
 *  Enum that decides what action to take if a problem is detected.
 */
var Action;
(function (Action) {
    /** Ignore the error and continue */
    Action["IGNORE"] = "";
    /** Warn about the error and continue */
    Action["WARN"] = "warn";
    /** Warn about the error and break */
    Action["ERROR"] = "error";
})(Action = exports.Action || (exports.Action = {}));
;
;
function base64(baseDir, options) {
    var opts = Object.assign({
        include: [],
        exclude: [/.*/],
        maxImageSize: 8192,
        maxReported: 10,
        actOnMissingFile: Action.ERROR,
        actOnLargeFile: Action.WARN,
        actOnEncodedTwice: Action.WARN,
        originalAsComment: false
    }, options);
    var cache = {};
    var reported = 0;
    var errors = 0;
    var warnings = 0;
    var wasted = 0;
    function error(value, action, url, msg) {
        msg = 'Image file ' + url + ' ' + msg;
        if (action === Action.ERROR) {
            errors++;
            console.error(msg);
        }
        else if (action === Action.WARN) {
            warnings++;
            if (reported++ < opts.maxReported) {
                console.warn(msg);
            }
        }
        return value;
    }
    function url(fn) {
        return func({
            url: function (path) {
                path = path.split('"').join('');
                path = path.split('\'').join('');
                var oldValue = path.trim();
                var newValue = fn.call(this, oldValue);
                if (!newValue) {
                    return 'url("' + oldValue + '")';
                }
                else if (opts.originalAsComment) {
                    return '/*' + oldValue + '*/ url("' + newValue + '")';
                }
                else {
                    return 'url("' + newValue + '")';
                }
            }
        }, false);
    }
    function matches(url) {
        return !url.startsWith('data:') &&
            (opts.include.some(function (re) { return !!url.match(re); }) || !opts.exclude.some(function (re) { return !!url.match(re); }));
    }
    function base64(url) {
        if (matches(url)) {
            if (cache[url]) {
                wasted += cache[url].length;
                return error(cache[url], opts.actOnEncodedTwice, url, "is encoded more than once");
            }
            else {
                var file = path.resolve(baseDir, url);
                if (fs.existsSync(file)) {
                    var data = new Datauri(file);
                    if (data.content.length < opts.maxImageSize) {
                        return cache[url] = data.content;
                    }
                    else {
                        return error(undefined, opts.actOnLargeFile, url, 'is too large');
                    }
                }
                else {
                    return error(undefined, opts.actOnMissingFile, url, 'is missing');
                }
            }
        }
        else {
            return undefined;
        }
    }
    return function () {
        var ret = url(base64).apply(this, arguments);
        if (warnings > 0 && errors > 0) {
            console.log(NAME + ": There were " + errors + " errors and " + warnings + " warnings");
        }
        else if (errors > 0) {
            console.log(NAME + ": There were " + errors + " errors");
        }
        else if (warnings > 0) {
            console.log(NAME + ": There were " + warnings + " warnings");
        }
        if (wasted > 0) {
            console.log(NAME + ": Duplicate images are using " + wasted + " bytes");
        }
        if (errors > 0) {
            throw new Error('There were errors, see the log');
        }
        else {
            return ret;
        }
    };
}
exports.base64 = base64;
//# sourceMappingURL=index.js.map